@@ -29,6 +29,7 @@ import org.apache.activemq.broker.ConsumerBrokerExchange;
 import org.apache.activemq.broker.DestinationAlreadyExistsException;
 import org.apache.activemq.broker.ProducerBrokerExchange;
 import org.apache.activemq.command.ActiveMQDestination;
+import org.apache.activemq.command.ConsumerControl;
 import org.apache.activemq.command.ConsumerId;
 import org.apache.activemq.command.ConsumerInfo;
 import org.apache.activemq.command.Message;
@@ -373,6 +374,7 @@ public abstract class AbstractRegion implements Region {
                     LOG.warn("Ack for non existent subscription, ack:" + ack);
                     throw new IllegalArgumentException("The subscription does not exist: " + ack.getConsumerId());
                 } else {
+                    LOG.debug("Ack for non existent subscription in recovery, ack:" + ack);
                     return;
                 }
             }
@@ -490,21 +492,40 @@ public abstract class AbstractRegion implements Region {
 
     /**
      * Removes a Producer.
-     * @param context the environment the operation is being executed under.
-     * @throws Exception TODO
+     * 
+     * @param context
+     *            the environment the operation is being executed under.
+     * @throws Exception
+     *             TODO
      */
-    public void removeProducer(ConnectionContext context, ProducerInfo info) throws Exception{
+    public void removeProducer(ConnectionContext context, ProducerInfo info) throws Exception {
         synchronized (destinationsMutex) {
             for (Iterator iter = destinationMap.get(info.getDestination()).iterator(); iter.hasNext();) {
-                Destination dest = (Destination)iter.next();
+                Destination dest = (Destination) iter.next();
                 dest.removeProducer(context, info);
             }
         }
     }
-    
-    protected void dispose(ConnectionContext context,Destination dest) throws Exception {
+
+    protected void dispose(ConnectionContext context, Destination dest) throws Exception {
         dest.dispose(context);
         dest.stop();
         destinationFactory.removeDestination(dest);
     }
+
+    public void processConsumerControl(ConsumerBrokerExchange consumerExchange, ConsumerControl control) {
+        Subscription sub = subscriptions.get(control.getConsumerId());
+        if (sub != null && sub instanceof AbstractSubscription) {
+            ((AbstractSubscription) sub).setPrefetchSize(control.getPrefetch());
+            if (LOG.isDebugEnabled()) {
+                LOG.debug("setting prefetch: " + control.getPrefetch() + ", on subscription: "
+                        + control.getConsumerId());
+            }
+            try {
+                lookup(consumerExchange.getConnectionContext(), control.getDestination(),false).wakeup();
+            } catch (Exception e) {
+                LOG.warn("failed to deliver consumerControl to destination: " + control.getDestination(), e);
+            }
+        }
+    }
 }
