@@ -16,15 +16,6 @@
  */
 package org.apache.activemq.transport.stomp;
 
-import org.apache.activemq.ActiveMQConnectionFactory;
-import org.apache.activemq.CombinationTestSupport;
-import org.apache.activemq.broker.BrokerFactory;
-import org.apache.activemq.broker.BrokerService;
-import org.apache.activemq.broker.jmx.BrokerViewMBean;
-import org.apache.activemq.command.ActiveMQQueue;
-import org.apache.activemq.command.ActiveMQTextMessage;
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
 import java.io.IOException;
 import java.net.Socket;
 import java.net.SocketTimeoutException;
@@ -32,6 +23,7 @@ import java.net.URI;
 import java.net.URISyntaxException;
 import java.net.UnknownHostException;
 import java.util.HashMap;
+import java.util.Map;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 import javax.jms.BytesMessage;
@@ -44,6 +36,15 @@ import javax.jms.ObjectMessage;
 import javax.jms.Session;
 import javax.jms.TextMessage;
 import javax.management.ObjectName;
+import org.apache.activemq.ActiveMQConnectionFactory;
+import org.apache.activemq.CombinationTestSupport;
+import org.apache.activemq.broker.BrokerFactory;
+import org.apache.activemq.broker.BrokerService;
+import org.apache.activemq.broker.jmx.BrokerViewMBean;
+import org.apache.activemq.command.ActiveMQQueue;
+import org.apache.activemq.command.ActiveMQTextMessage;
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.LogFactory;
 
 public class StompTest extends CombinationTestSupport {
     private static final Log LOG = LogFactory.getLog(StompTest.class);
@@ -58,7 +59,7 @@ public class StompTest extends CombinationTestSupport {
     private Connection connection;
     private Session session;
     private ActiveMQQueue queue;
-    private String xmlObject = "<pojo>\n"
+    private final String xmlObject = "<pojo>\n"
             + "  <name>Dejan</name>\n"
             + "  <city>Belgrade</city>\n"
             + "</pojo>";
@@ -74,7 +75,7 @@ public class StompTest extends CombinationTestSupport {
         + "  </entry>\n"
         + "</map>\n";
 
-    private String jsonObject = "{\"pojo\":{"
+    private final String jsonObject = "{\"pojo\":{"
         + "\"name\":\"Dejan\","
         + "\"city\":\"Belgrade\""
         + "}}";
@@ -86,6 +87,7 @@ public class StompTest extends CombinationTestSupport {
         + "]"
         + "}}";
 
+    @Override
     protected void setUp() throws Exception {
         // The order of the entries is different when using ibm jdk 5.
         if (System.getProperty("java.vendor").equals("IBM Corporation")
@@ -132,6 +134,7 @@ public class StompTest extends CombinationTestSupport {
         return getClass().getName() + "." + getName();
     }
 
+    @Override
     protected void tearDown() throws Exception {
         try {
             connection.close();
@@ -244,6 +247,26 @@ public class StompTest extends CombinationTestSupport {
         assertEquals("bar", "123", message.getStringProperty("bar"));
     }
 
+    public void testSendMessageWithDelay() throws Exception {
+
+        MessageConsumer consumer = session.createConsumer(queue);
+
+        String frame = "CONNECT\n" + "login: system\n" + "passcode: manager\n\n" + Stomp.NULL;
+        stompConnection.sendFrame(frame);
+
+        frame = stompConnection.receiveFrame();
+        assertTrue(frame.startsWith("CONNECTED"));
+
+        frame = "SEND\n" + "AMQ_SCHEDULED_DELAY:5000\n"  + "destination:/queue/" + getQueueName() + "\n\n" + "Hello World" + Stomp.NULL;
+
+        stompConnection.sendFrame(frame);
+
+        TextMessage message = (TextMessage)consumer.receive(2000);
+        assertNull(message);
+        message = (TextMessage)consumer.receive(5000);
+        assertNotNull(message);
+    }
+
     public void testSendMessageWithStandardHeaders() throws Exception {
 
         MessageConsumer consumer = session.createConsumer(queue);
@@ -273,6 +296,59 @@ public class StompTest extends CombinationTestSupport {
         assertEquals("GroupID", "abc", amqMessage.getGroupID());
     }
 
+    public void testReceipts() throws Exception {
+
+        StompConnection receiver = new StompConnection();
+        URI connectUri = new URI(bindAddress);
+        receiver.open(createSocket(connectUri));
+        String frame = "CONNECT\n" + "login: system\n" + "passcode: manager\n\n" + Stomp.NULL;
+        receiver.sendFrame(frame);
+
+        frame = receiver.receiveFrame();
+        assertTrue(frame.startsWith("CONNECTED"));
+
+        frame = "SUBSCRIBE\n" + "destination:/queue/" + getQueueName() + "\n" + "ack:auto\n\n" + Stomp.NULL;
+        receiver.sendFrame(frame);
+
+        frame = "CONNECT\n" + "login: system\n" + "passcode: manager\n\n" + Stomp.NULL;
+        stompConnection.sendFrame(frame);
+
+        frame = stompConnection.receiveFrame();
+        assertTrue(frame.startsWith("CONNECTED"));
+
+        frame = "SEND\n" + "destination:/queue/" + getQueueName() + "\n" + "receipt: msg-1\n" + "\n\n" + "Hello World" + Stomp.NULL;
+        stompConnection.sendFrame(frame);
+
+        frame = receiver.receiveFrame();
+        assertTrue(frame.startsWith("MESSAGE"));
+        assertTrue("Stomp Message does not contain receipt request", frame.indexOf(Stomp.Headers.RECEIPT_REQUESTED) == -1);
+
+        frame = stompConnection.receiveFrame();
+        assertTrue(frame.startsWith("RECEIPT"));
+        assertTrue("Receipt contains correct receipt-id", frame.indexOf(Stomp.Headers.Response.RECEIPT_ID) >= 0);
+
+        frame = "DISCONNECT\n" + "\n\n" + Stomp.NULL;
+        receiver.sendFrame(frame);
+
+        waitForFrameToTakeEffect();
+
+        MessageConsumer consumer = session.createConsumer(queue);
+
+        frame = "SEND\n" + "destination:/queue/" + getQueueName() + "\n" + "receipt: msg-1\n" + "\n\n" + "Hello World" + Stomp.NULL;
+        stompConnection.sendFrame(frame);
+
+        frame = stompConnection.receiveFrame();
+        assertTrue(frame.startsWith("RECEIPT"));
+        assertTrue("Receipt contains correct receipt-id", frame.indexOf(Stomp.Headers.Response.RECEIPT_ID) >= 0);
+
+        TextMessage message = (TextMessage)consumer.receive(2500);
+        assertNotNull(message);
+        assertNull("JMS Message does not contain receipt request", message.getStringProperty(Stomp.Headers.RECEIPT_REQUESTED));
+
+        frame = "DISCONNECT\n" + "\n\n" + Stomp.NULL;
+        stompConnection.sendFrame(frame);
+    }
+
     public void testSubscribeWithAutoAck() throws Exception {
 
         String frame = "CONNECT\n" + "login: system\n" + "passcode: manager\n\n" + Stomp.NULL;
@@ -322,6 +398,64 @@ public class StompTest extends CombinationTestSupport {
         stompConnection.sendFrame(frame);
     }
 
+    public void testBytesMessageWithNulls() throws Exception {
+
+        String frame = "CONNECT\n" + "login: system\n" + "passcode: manager\n\n" + Stomp.NULL;
+        stompConnection.sendFrame(frame);
+
+        frame = stompConnection.receiveFrame();
+        assertTrue(frame.startsWith("CONNECTED"));
+
+        frame = "SEND\n destination:/queue/" + getQueueName() + "\ncontent-length:5" + " \n\n" + "\u0001\u0002\u0000\u0004\u0005" + Stomp.NULL;
+        stompConnection.sendFrame(frame);
+
+        frame = "SUBSCRIBE\n" + "destination:/queue/" + getQueueName() + "\n" + "ack:auto\n\n" + Stomp.NULL;
+        stompConnection.sendFrame(frame);
+
+        StompFrame message = stompConnection.receive();
+        assertTrue(message.getAction().startsWith("MESSAGE"));
+
+        String length = message.getHeaders().get("content-length");
+        assertEquals("5", length);
+
+        assertEquals(5, message.getContent().length);
+
+        frame = "DISCONNECT\n" + "\n\n" + Stomp.NULL;
+        stompConnection.sendFrame(frame);
+    }
+
+    public void testSendMultipleBytesMessages() throws Exception {
+
+    	final int MSG_COUNT = 50;
+
+        String frame = "CONNECT\n" + "login: system\n" + "passcode: manager\n\n" + Stomp.NULL;
+        stompConnection.sendFrame(frame);
+
+        frame = stompConnection.receiveFrame();
+        assertTrue(frame.startsWith("CONNECTED"));
+
+        for( int ix = 0; ix < MSG_COUNT; ix++) {
+            frame = "SEND\n destination:/queue/" + getQueueName() + "\ncontent-length:5" + " \n\n" + "\u0001\u0002\u0000\u0004\u0005" + Stomp.NULL;
+            stompConnection.sendFrame(frame);
+        }
+
+        frame = "SUBSCRIBE\n" + "destination:/queue/" + getQueueName() + "\n" + "ack:auto\n\n" + Stomp.NULL;
+        stompConnection.sendFrame(frame);
+
+        for( int ix = 0; ix < MSG_COUNT; ix++) {
+            StompFrame message = stompConnection.receive();
+            assertTrue(message.getAction().startsWith("MESSAGE"));
+
+            String length = message.getHeaders().get("content-length");
+            assertEquals("5", length);
+
+            assertEquals(5, message.getContent().length);
+        }
+
+        frame = "DISCONNECT\n" + "\n\n" + Stomp.NULL;
+        stompConnection.sendFrame(frame);
+    }
+
     public void testSubscribeWithMessageSentWithProperties() throws Exception {
 
         String frame = "CONNECT\n" + "login: system\n" + "passcode: manager\n\n" + Stomp.NULL;
@@ -748,6 +882,165 @@ public class StompTest extends CombinationTestSupport {
         stompConnection.sendFrame(frame);
     }
 
+    public void testTransformationReceiveObject() throws Exception {
+
+        MessageProducer producer = session.createProducer(new ActiveMQQueue("USERS." + getQueueName()));
+        ObjectMessage message = session.createObjectMessage(new SamplePojo("Dejan", "Belgrade"));
+        producer.send(message);
+
+        String frame = "CONNECT\n" + "login: system\n" + "passcode: manager\n\n" + Stomp.NULL;
+        stompConnection.sendFrame(frame);
+
+        frame = stompConnection.receiveFrame();
+        assertTrue(frame.startsWith("CONNECTED"));
+
+        frame = "SUBSCRIBE\n" + "destination:/queue/USERS." + getQueueName() + "\n" + "ack:auto" + "\n" + "transformation:"	+ Stomp.Transformations.JMS_OBJECT_XML + "\n\n" + Stomp.NULL;
+        stompConnection.sendFrame(frame);
+
+        frame = stompConnection.receiveFrame();
+
+        assertTrue(frame.trim().endsWith(xmlObject));
+
+        frame = "DISCONNECT\n" + "\n\n" + Stomp.NULL;
+        stompConnection.sendFrame(frame);
+    }
+
+    public void testTransformationReceiveXMLObjectAndMap() throws Exception {
+        MessageProducer producer = session.createProducer(new ActiveMQQueue("USERS." + getQueueName()));
+        ObjectMessage objMessage = session.createObjectMessage(new SamplePojo("Dejan", "Belgrade"));
+        producer.send(objMessage);
+
+        MapMessage mapMessage = session.createMapMessage();
+        mapMessage.setString("name", "Dejan");
+        mapMessage.setString("city", "Belgrade");
+        producer.send(mapMessage);
+
+        String frame = "CONNECT\n" + "login: system\n" + "passcode: manager\n\n" + Stomp.NULL;
+        stompConnection.sendFrame(frame);
+
+        frame = stompConnection.receiveFrame();
+        assertTrue(frame.startsWith("CONNECTED"));
+
+        frame = "SUBSCRIBE\n" + "destination:/queue/USERS." + getQueueName() + "\n" + "ack:auto" + "\n" + "transformation:"	+ Stomp.Transformations.JMS_XML + "\n\n" + Stomp.NULL;
+        stompConnection.sendFrame(frame);
+
+        frame = stompConnection.receiveFrame();
+
+        assertTrue(frame.trim().endsWith(xmlObject));
+
+        frame = stompConnection.receiveFrame();
+
+        assertTrue(frame.trim().endsWith(xmlMap.trim()));
+
+        frame = "DISCONNECT\n" + "\n\n" + Stomp.NULL;
+        stompConnection.sendFrame(frame);
+    }
+
+    public void testTransformationReceiveJSONObjectAndMap() throws Exception {
+        MessageProducer producer = session.createProducer(new ActiveMQQueue("USERS." + getQueueName()));
+        ObjectMessage objMessage = session.createObjectMessage(new SamplePojo("Dejan", "Belgrade"));
+        producer.send(objMessage);
+
+        MapMessage mapMessage = session.createMapMessage();
+        mapMessage.setString("name", "Dejan");
+        mapMessage.setString("city", "Belgrade");
+        producer.send(mapMessage);
+
+        String frame = "CONNECT\n" + "login: system\n" + "passcode: manager\n\n" + Stomp.NULL;
+        stompConnection.sendFrame(frame);
+
+        frame = stompConnection.receiveFrame();
+        assertTrue(frame.startsWith("CONNECTED"));
+
+        frame = "SUBSCRIBE\n" + "destination:/queue/USERS." + getQueueName() + "\n" + "ack:auto" + "\n" + "transformation:"	+ Stomp.Transformations.JMS_JSON + "\n\n" + Stomp.NULL;
+        stompConnection.sendFrame(frame);
+
+        frame = stompConnection.receiveFrame();
+
+        assertTrue(frame.trim().endsWith(jsonObject));
+
+        frame = stompConnection.receiveFrame();
+
+        assertTrue(frame.trim().endsWith(jsonMap.trim()));
+
+        frame = "DISCONNECT\n" + "\n\n" + Stomp.NULL;
+        stompConnection.sendFrame(frame);
+    }
+
+    public void testTransformationSendAndReceiveXmlMap() throws Exception {
+
+        String frame = "CONNECT\n" + "login: system\n" + "passcode: manager\n\n" + Stomp.NULL;
+        stompConnection.sendFrame(frame);
+
+        frame = stompConnection.receiveFrame();
+        assertTrue(frame.startsWith("CONNECTED"));
+
+        frame = "SUBSCRIBE\n" + "destination:/queue/" + getQueueName() + "\n" + "ack:auto" + "\n" + "transformation:" + Stomp.Transformations.JMS_XML + "\n\n" + Stomp.NULL;
+        stompConnection.sendFrame(frame);
+
+        frame = "SEND\n" + "destination:/queue/" + getQueueName() + "\n" + "transformation:" + Stomp.Transformations.JMS_MAP_JSON + "\n\n" + jsonMap + Stomp.NULL;
+
+        stompConnection.sendFrame(frame);
+
+        frame = stompConnection.receiveFrame();
+
+        assertNotNull(frame);
+        assertTrue(frame.trim().endsWith(xmlMap.trim()));
+        assertTrue(frame.contains("jms-map-xml"));
+    }
+
+    public void testTransformationSendAndReceiveJsonMap() throws Exception {
+
+        String frame = "CONNECT\n" + "login: system\n" + "passcode: manager\n\n" + Stomp.NULL;
+        stompConnection.sendFrame(frame);
+
+        frame = stompConnection.receiveFrame();
+        assertTrue(frame.startsWith("CONNECTED"));
+
+        frame = "SUBSCRIBE\n" + "destination:/queue/" + getQueueName() + "\n" + "ack:auto" + "\n" + "transformation:" + Stomp.Transformations.JMS_JSON + "\n\n" + Stomp.NULL;
+        stompConnection.sendFrame(frame);
+
+        frame = "SEND\n" + "destination:/queue/" + getQueueName() + "\n" + "transformation:" + Stomp.Transformations.JMS_MAP_XML + "\n\n" + xmlMap + Stomp.NULL;
+
+        stompConnection.sendFrame(frame);
+
+        frame = stompConnection.receiveFrame();
+
+        assertNotNull(frame);
+        assertTrue(frame.trim().endsWith(jsonMap.trim()));
+        assertTrue(frame.contains("jms-map-json"));
+    }
+
+    public void testTransformationReceiveBytesMessage() throws Exception {
+
+        MessageProducer producer = session.createProducer(new ActiveMQQueue("USERS." + getQueueName()));
+        BytesMessage message = session.createBytesMessage();
+        message.writeBytes(new byte[]{1, 2, 3, 4, 5});
+        producer.send(message);
+
+        String frame = "CONNECT\n" + "login: system\n" + "passcode: manager\n\n" + Stomp.NULL;
+        stompConnection.sendFrame(frame);
+
+        frame = stompConnection.receiveFrame();
+        assertTrue(frame.startsWith("CONNECTED"));
+
+        frame = "SUBSCRIBE\n" + "destination:/queue/USERS." + getQueueName() + "\n" + "ack:auto" + "\n" + "transformation:"	+ Stomp.Transformations.JMS_XML + "\n\n" + Stomp.NULL;
+        stompConnection.sendFrame(frame);
+
+        frame = stompConnection.receiveFrame();
+        assertTrue(frame.startsWith("MESSAGE"));
+
+        Pattern cl = Pattern.compile("Content-length:\\s*(\\d+)", Pattern.CASE_INSENSITIVE);
+        Matcher clMmatcher = cl.matcher(frame);
+        assertTrue(clMmatcher.find());
+        assertEquals("5", clMmatcher.group(1));
+
+        assertFalse(Pattern.compile("type:\\s*null", Pattern.CASE_INSENSITIVE).matcher(frame).find());
+
+        frame = "DISCONNECT\n" + "\n\n" + Stomp.NULL;
+        stompConnection.sendFrame(frame);
+    }
+
     public void testTransformationNotOverrideSubscription() throws Exception {
         MessageProducer producer = session.createProducer(new ActiveMQQueue("USERS." + getQueueName()));
         ObjectMessage message = session.createObjectMessage(new SamplePojo("Dejan", "Belgrade"));
@@ -1086,6 +1379,36 @@ public class StompTest extends CombinationTestSupport {
         assertEquals("system", message.getHeaders().get(Stomp.Headers.Message.USERID));
     }
 
+    public void testClientSetMessageIdIsIgnored() throws Exception {
+        HashMap<String, String> headers = new HashMap<String, String>();
+        headers.put(Stomp.Headers.Message.MESSAGE_ID, "Thisisnotallowed");
+        headers.put(Stomp.Headers.Message.TIMESTAMP, "1234");
+        headers.put(Stomp.Headers.Message.REDELIVERED, "true");
+        headers.put(Stomp.Headers.Message.SUBSCRIPTION, "Thisisnotallowed");
+        headers.put(Stomp.Headers.Message.USERID, "Thisisnotallowed");
+
+
+        stompConnection.connect("system", "manager");
+
+
+        stompConnection.send("/queue/" + getQueueName(), "msg", null, headers);
+
+        stompConnection.subscribe("/queue/" + getQueueName());
+        StompFrame stompMessage = stompConnection.receive();
+
+        Map<String, String> mess_headers = new HashMap<String, String>();
+        mess_headers = stompMessage.getHeaders();
+
+        assertFalse("Thisisnotallowed".equals(mess_headers.get(Stomp.Headers.Message.MESSAGE_ID)
+                ));
+        assertFalse("1234".equals(mess_headers.get(Stomp.Headers.Message.TIMESTAMP)));
+        assertNull(mess_headers.get(Stomp.Headers.Message.REDELIVERED));
+        assertNull(mess_headers.get(Stomp.Headers.Message.SUBSCRIPTION));
+        assertEquals("system", mess_headers.get(Stomp.Headers.Message.USERID));
+
+    }
+
+
     protected void assertClients(int expected) throws Exception {
         org.apache.activemq.broker.Connection[] clients = broker.getBroker().getClients();
         int actual = clients.length;
